@@ -1,6 +1,6 @@
 /*
- * This file is part of SpyBlock <http://www.gnu.org/software/gnuzilla/>,
- * Copyright (C) 2006-2014 Eyeo GmbH (Adblock Plus) 
+ * This file is part of SpyBlock <https://www.gnu.org/software/gnuzilla/>,
+ * Copyright (C) 2006-2015 Eyeo GmbH (Adblock Plus) 
  * Copyright (C) 2015 Ruben Rodriguez <ruben@gnu.org>
  *
  * SpyBlock is free software: you can redistribute it and/or modify
@@ -237,7 +237,7 @@ var SubscriptionActions =
       else if (accelKey == Ci.nsIDOMKeyEvent.DOM_VK_ALT)
         result = this._altMask;
     } catch(e) {}
-    this.__defineGetter__("_accelMask", function() result);
+    Object.defineProperty(this, "_accelMask", {value: result});
     return result;
   },
 
@@ -445,7 +445,7 @@ var TitleEditor =
     if (save)
     {
       newTitle = subscriptionNode.getElementsByClassName("titleEditor")[0].value;
-      newTitle = newTitle.replace(/^\s+/, "").replace(/\s+$/, "");
+      newTitle = newTitle.trim();
     }
 
     let subscription = Templater.getDataForNode(subscriptionNode).subscription
